#ifndef ACTORHEADER
#define ACTORHEADER
#include "RenderableObject.h"
#include "Matrix4x4.h"
#include "Vector3.h"
#include "SkelChar.h"
#include "Quaternion.h"
#include "KeyFrameAnim.h"
#include <vector>
#include <string>
#include "TimeStamp.h"
#include "Material.h"
#include "Repetoire.h"
#include <set>

#include <Pose.h>

class Actor: public RenderableObject
{

	//our skeletal character
	std::string skelName;
	int skelInst;
	bool skelSet;

	//our last static pose
	Pose savedPose;

	Pose currentPose;
	Pose lastPose;

	//the name of our material
	std::string matName;
	
	
	typedef struct
	{
		std::string name;
		TimeStamp startTime;
		Matrix4x4 startxform;
		
		float starthead;
		Vector3 startPos;
	} animPlay;

	//our currently playing animation
	animPlay a1, a2;

	
	//what we are currently doing	
	bool playing;
	bool blending;
	bool stillBlend;

	Vector3 translation;
	
	float defaultWindow;
	Repetoire * rep;
	std::map<std::string,float> attributes;

	float heading;
public:

	Actor();
	~Actor();
	bool SetSkel(const char * skelName);
	void SetHeading(float h);

	bool PlayAnim(const char * name);
	bool IsIdle();
	bool LoadXML(const char * name);
	void SetMaterial(const char * mame);
	void Draw();
	void Update();
	void SetPose(Pose & p);
	void GetJointXform(const char * name, Matrix4x4 & m);
	bool PlayAction(const char * actionname);
	void SetAttribute(const char * name, float val);
	std::string GetCurrentAnimation();
	float GetHeading();
	float GetAttribute(const char * name);
};



#endif 