#ifndef AGENTHEADER
#define AGENTHEADER

#include "Actor.h"
#include "InputEvent.h"
#include "BehaviorRoutines.h"
#include "Perception.h"
#include "State.h"
#include "Enums.h"
#include <vector>
#include <set>


class Agent
{
protected:
	Actor actor;
	std::vector<InputEvent> events;
	Agent * opponent;
	std::vector<std::string> thoughts;
	std::string currentaction;
	BehaviorRoutines br;
	Perception perception;
	State state;
	float debugcolorr, debugcolorg, debugcolorb;
	bool actioninterruptible;
	TimeStamp timer;

public:
	Agent();
	~Agent();
	bool LoadActor(const char * filename);
	bool LoadProfile(const char * filename);
	bool PlayAction(const char * actionname, bool override = false);
	void GetPosition(Vector3 & pos);
	void SetPosition(const Vector3 & pos);
	void SetHeading(float h);
	void AddInputEvent(InputEvent & event);
	void UpdateAnim();
	void Render();
	void SetOpponent(Agent * op);
	void DebugRender();
	void KeepInRing(float bounds);
	void Alert(const char * message);
	void HandleCollision(std::string myjoint, std::string opjoint);
	virtual void ScoreAction(ActionTerminator at) = 0;
	virtual void Think() = 0;

	std::string GetThoughts(const unsigned int index);
	Actor * GetActor();
	Agent * GetOpponent();
	Perception * GetPerception();
	std::string GetAction();
	State * GetState();
};

#endif