#include "BoxingSim.h"

#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif

BoxingSim::BoxingSim()
{
		RING_SIZE = 4;
}

BoxingSim::~BoxingSim()
{
}

bool 
BoxingSim::Initialize()
{
	initTime = TimeStamp();

	theta = -90;
	phi = 45;
	dist = 10;
	theCamera.SetDist(dist);
	theCamera.SetOrbitAngle(theta,phi);
	theCamera.Orbit();
	overheadLight.SetPos(0,20,0);
	frontLight.SetPos(0,0,5);
	frontLight.SetAmbient(.5,.5,.5);
	Font::LoadFont("Tahoma.bmp","tahoma.dat","Tahoma",false);
	updates = 0;

	//load agent actors
	if (!boxer1.LoadActor("boxer1.xml"))
		return false;
	boxer1.SetPosition(Vector3(2,0,2));
	boxer1.SetHeading(3.14159f);
	
	if (!boxer2.LoadActor("boxer2.xml"))
		return false;
	boxer2.SetPosition(Vector3(-2,0,-2));
	boxer2.SetHeading(0);

	if (!boxer1.LoadProfile("profile1.xml"))
		return false;
	if (!boxer2.LoadProfile("profile2.xml"))
		return false;

	if (!boxer1.LoadIGAdjustments("boxer1.adj"))
		printf("Unable to load adjustments for boxer 1, using defaults\n");
#ifdef TWOREACTIVE
	if (!boxer2.LoadIGAdjustments("boxer2.adj"))
		printf("Unable to load adjustments for boxer 2, using defaults\n");
#endif

	boxer1.SetOpponent(&boxer2);
	boxer2.SetOpponent(&boxer1);

	return true;
}

void 
BoxingSim::RenderPersp()
{
	TimeStamp t1;
	glEnable(GL_LIGHTING);
	
	//look through the default camera
	theCamera.SetView();

	frontLight.Switch(true);
	overheadLight.Switch(true);

	//draw the world coordinate axis
	glPushMatrix(); glTranslatef(RING_SIZE,0,RING_SIZE); glRotatef(180,0,1,0);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(-RING_SIZE,0,RING_SIZE); glRotatef(90,0,1,0);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(RING_SIZE,0,-RING_SIZE); glRotatef(-90,0,1,0);
	Axis3D::StaticDraw(); glPopMatrix();

	glPushMatrix(); glTranslatef(-RING_SIZE,0,-RING_SIZE);
	Axis3D::StaticDraw(); glPopMatrix();

	glColor3f(.0f,.1f,.2f);
	glBegin(GL_QUADS);
	glNormal3f(0,1,0);
	glVertex3f(-RING_SIZE,-0.14f,RING_SIZE);
	glVertex3f(RING_SIZE,-0.14f,RING_SIZE);
	glVertex3f(RING_SIZE,-0.14f,-RING_SIZE);
	glVertex3f(-RING_SIZE,-0.14f,-RING_SIZE);
	glEnd();

	boxer1.Render();
	boxer2.Render();

//    boxer1.DebugRender();
//    boxer2.DebugRender();

	glDisable(GL_LIGHTING);
	renderTime = t1.Interval(TimeStamp());


}

void 
BoxingSim::RenderOrtho()
{
	static TimeStamp last;
	
	glTranslatef(10,24,0);
	glColor3f(1,1,1);

	Font::Printf("Tahoma",30,false,"FPS:%3.2f",1.0f/last.Interval(TimeStamp()));

	glLoadIdentity();
	glTranslatef(0,(float)windowY-30*4,0);
	Font::Printf("Tahoma",30,false,"Reactive (orange)");
	glTranslatef(0,30,0);
	for (unsigned int i = 0; i < 3; i++)
	{
		Font::Printf("Tahoma",30,false,boxer1.GetThoughts(i).c_str());
		glTranslatef(0,30,0);
	}

	glLoadIdentity();
	glTranslatef((float)windowX-200,(float)windowY-30*4,0);
#ifdef TWOREACTIVE
	Font::Printf("Tahoma",30,false,"Reactive (silver)");
#else
	Font::Printf("Tahoma",30,false,"Keyboard (silver)");
#endif
	glTranslatef(0,30,0);
	for (unsigned int i = 0; i < 3; i++)
	{
		Font::Printf("Tahoma",30,false,boxer2.GetThoughts(i).c_str());
		glTranslatef(0,30,0);
	}

	last.Reset();
}

void
BoxingSim::HandleInput()
{
	std::set<InputEvent>::iterator it;
	for (it = events.begin(); it != events.end(); it++)
	{
		switch(it->type)
		{
		case InputEvent::EventType::KeyDown:
			switch(it->ID)
			{
			case 'q':
				exit(1);
				break;
			case '/':
				boxer1.ResetIGAdjustments();
#ifdef TWOREACTIVE
				boxer2.ResetIGAdjustments();
#endif
				printf("Adjustments reset\n");
				break;
			case '9': case '-':
				boxer1.AddInputEvent(*it);
				break;
			default:
				boxer2.AddInputEvent(*it);
				break;
			}
			break;
		case InputEvent::EventType::MouseDrag:
			theta += it->coordinate.Get(0)*.01f;
			phi += it->coordinate.Get(1)*.01f;
			dist += it->coordinate.Get(2)*.1f;
			theCamera.SetOrbitAngle(theta,phi);
			theCamera.SetDist(dist);
			theCamera.Orbit();
			break;

		default:
			break;

		}
	}
	events.clear();
}


void 
BoxingSim::Update()
{
	HandleInput();

	TimeStamp t1;
	interval = lastUpdate.Interval(t1);
	lastUpdate = t1;
	updates++;

	boxer1.Think();
	boxer2.Think();

	boxer1.UpdateAnim();
	boxer2.UpdateAnim();

	CollisionDetection();

	theCamera.SetOrbitAngle((float) t1.Msec() / 1000.0f, 1.0f);
	
	updateTime = t1.Interval(TimeStamp());
}

void
BoxingSim::CollisionDetection()
{
	Vector3 b1pos, b2pos, diffinpos;
	float separation;

	boxer1.GetPosition(b1pos);
	boxer2.GetPosition(b2pos);
	diffinpos.Sub(b1pos, b2pos);
	separation = diffinpos.Magnitude();

	if (separation < 1.5f)
	{
		if (separation < 0.75f)
		{
			Vector3 midpoint(0,0,0);

			if (separation < 0.0001)
				diffinpos.Set(1,0,0);

			midpoint.Add(diffinpos);
			midpoint.Mult(0.5f);
			midpoint.Add(b2pos);

			diffinpos.Normalize();
			diffinpos.Mult(0.375f);

			b1pos.Set(0,0,0);
			b1pos.Add(midpoint);
			b1pos.Add(diffinpos);
			b2pos.Set(0,0,0);
			b2pos.Add(midpoint);
			b2pos.Sub(diffinpos);

			boxer1.SetPosition(b1pos);
			boxer2.SetPosition(b2pos);
		}

		float blocksize;
		if (boxer2.GetAction() == "block")
			blocksize = 0.075f;
		else
			blocksize = 0.05f;

		if (boxer1.GetAction() == "jab")
		{
			CheckCollision("lhand", "head", 0.05f, 0.2f);
			CheckCollision("lhand", "lclavicle", 0.05f, 0.175f);
			CheckCollision("lhand", "lhumerus", 0.05f, 0.1f);
			CheckCollision("lhand", "rhumerus", 0.05f, 0.1f);

			CheckCollision("lhand", "lhand", 0.05f, blocksize);
			CheckCollision("lhand", "rhand", 0.05f, blocksize);
			CheckCollision("lhand", "lradius", 0.05f, blocksize);
			CheckCollision("lhand", "rradius", 0.05f, blocksize);
		}
		else if (boxer1.GetAction() == "cross" ||
		         boxer1.GetAction() == "hook" ||
		         boxer1.GetAction() == "upper_cut")
		{
			CheckCollision("rhand", "head", 0.05f, 0.2f);
			CheckCollision("rhand", "lclavicle", 0.05f, 0.175f);
			CheckCollision("rhand", "lhumerus", 0.05f, 0.1f);
			CheckCollision("rhand", "rhumerus", 0.05f, 0.1f);
			CheckCollision("rhand", "thorax", 0.05f, 0.2f);

			CheckCollision("rhand", "lhand", 0.05f, blocksize);
			CheckCollision("rhand", "rhand", 0.05f, blocksize);
			CheckCollision("rhand", "lradius", 0.05f, blocksize);
			CheckCollision("rhand", "rradius", 0.05f, blocksize);
		}

		if (boxer1.GetAction() == "block")
			blocksize = 0.075f;
		else
			blocksize = 0.05f;

		if (boxer2.GetAction() == "jab")
		{
			CheckCollision("head", "lhand", 0.2f, 0.05f);
			CheckCollision("lclavicle", "lhand", 0.175f, 0.05f);
			CheckCollision("lhumerus", "lhand", 0.1f, 0.05f);
			CheckCollision("rhumerus", "lhand", 0.1f, 0.05f);

			CheckCollision("lhand", "lhand", 0.05f, blocksize);
			CheckCollision("rhand", "lhand", 0.05f, blocksize);
			CheckCollision("lradius", "lhand", 0.05f, blocksize);
			CheckCollision("rradius", "lhand", 0.05f, blocksize);
		}
		else if (boxer2.GetAction() == "cross" ||
		         boxer2.GetAction() == "hook" ||
		         boxer2.GetAction() == "upper_cut")
		{
			CheckCollision("head", "rhand", 0.2f, 0.05f);
			CheckCollision("lclavicle", "rhand", 0.175f, 0.05f);
			CheckCollision("lhumerus", "rhand", 0.1f, 0.05f);
			CheckCollision("rhumerus", "rhand", 0.1f, 0.05f);
			CheckCollision("thorax", "rhand", 0.2f, 0.05f);

			CheckCollision("lhand", "rhand", 0.05f, blocksize);
			CheckCollision("rhand", "rhand", 0.05f, blocksize);
			CheckCollision("lradius", "rhand", 0.05f, blocksize);
			CheckCollision("rradius", "rhand", 0.05f, blocksize);
		}
	}

	boxer1.KeepInRing(RING_SIZE);
	boxer2.KeepInRing(RING_SIZE);
}

void
BoxingSim::CheckCollision(const char * b1joint, const char * b2joint,
                          float radius1, float radius2)
{
	if (JointDistance(b1joint, b2joint) - (radius1 + radius2) < 0)
	{
		boxer1.HandleCollision(b1joint, b2joint);
		boxer2.HandleCollision(b2joint, b1joint);
	}
}

float
BoxingSim::JointDistance(const char * b1joint, const char * b2joint)
{
	Matrix4x4 j1matrix, j2matrix;
	Vector3 j1pos(0,0,0), j2pos(0,0,0);

	boxer1.GetActor()->GetJointXform(b1joint, j1matrix);
	boxer2.GetActor()->GetJointXform(b2joint, j2matrix);

	j1pos.Mult(j1matrix);
	j2pos.Mult(j2matrix);

	return j1pos.Distance(j2pos);
}