#include "Camera.h"
//#include "MatrixVector.h"
#include "Matrix4x4.h"
#include "Vector3.h"
#include <valarray>

Camera::Camera()
{
	printf("Created new Camera id %d \n",objID);
	centre.Set(0,0,0);
	dist = 5.0f;
	theta = phi = 0.0f;
}

Camera::~Camera()
{
}

void 
Camera::SetCentre(const Vector3 & targ)
{
	centre = targ;
}

void 
Camera::SetDist(const float d)
{
	if (d> 0.0f)
		dist = d;
}

void 
Camera::SetOrbitAngle(const float t,const float p)
{
	float maxphi = 3.14159f/2.0f;
	phi = (p > maxphi) ? maxphi : p;
	phi = (p < 0) ? 0 : p;

	theta = t;

}

void Camera::Orbit()
{
	Pos.Set(centre.Get(0) + dist*cosf(theta)*cosf(phi),
			centre.Get(1) + dist*sinf(phi),
			centre.Get(2)+ dist*sinf(theta)*cosf(phi));
	
	Matrix4x4 m1,m2;
	m1.RotateY(3.14159f/2.0f-theta);
	m2.RotateX(-phi);
	m1.PostMult(m2);
	Angle.setMatrix(m1);

}
void Camera::SetView()
{
	glLoadIdentity();
	Matrix4x4 m;
	GetFullXformInv(m);
	m.Transpose();
	glMultMatrixf(m.GetPtr());
	//ApplyInverseTransform();
}


void Camera::Draw()
{


	glRotatef(90,0,1,0);
	glBegin(GL_QUADS);
	glColor3f(.5,.5,.5);

	glNormal3f(0,0,1);
	glVertex3f(.3f,.25f,.25);
	glVertex3f(-.4f,.25f,.25);
	glVertex3f(-.4f,-.25f,.25);
	glVertex3f(.3f,-.25f,.25);

	glNormal3f(0,0,-1);
	glVertex3f(.3f,.25f,-.25);
	glVertex3f(.3f,-.25f,-.25);
	glVertex3f(-.4f,-.25f,-.25);
	glVertex3f(-.4f,.25f,-.25);

	glNormal3f(0,1,0);
	glVertex3f(.3f,.25f,-.25);
	glVertex3f(-.4f,.25f,-.25);
	glVertex3f(-.4f,.25f,.25);
	glVertex3f(.3f,.25f,.25);

	glNormal3f(0,-1,0);
	glVertex3f(.3f,-.25f,-.25);
	glVertex3f(.3f,-.25f,.25);
	glVertex3f(-.4f,-.25f,.25);
	glVertex3f(-.4f,-.25f,-.25);


	glNormal3f(1,0,0);
	glVertex3f(.3f,.25f,-.25f);
	glVertex3f(.3f,.25f,.25f);
	glVertex3f(.3f,-.25f,.25f);
	glVertex3f(.3f,-.25f,-.25f);

	glNormal3f(-1,0,0);
	glVertex3f(-.4f,.25f,-.25f);
	glVertex3f(-.4f,-.25f,-.25f);
	glVertex3f(-.4f,-.25f,.25f);
	glVertex3f(-.4f,.25f,.25f);

	
	glColor3f(.4f,.4f,.6f);
	glNormal3f(1,0,0);
	glVertex3f(.6f,.25f,-.25f);
	glVertex3f(.6f,.25f,.25f);
	glVertex3f(.6f,-.25f,.25f);
	glVertex3f(.6f,-.25f,-.25f);


	glEnd();


	glColor3f(.25,.25,.25);

	glBegin(GL_TRIANGLES);
	
	glNormal3f(-.385f,0,.923f);
	glVertex3f(.6f,.25f,.25f);
	glVertex3f(0,0,0);
	glVertex3f(.6f,-.25f,.25f);

	glNormal3f(-.385f,.923f,0);
	glVertex3f(.6f,.25f,-.25f);
	glVertex3f(0,0,0);
	glVertex3f(.6f,.25f,.25f);

	glNormal3f(-.385f,0,-.923f);
	glVertex3f(.6f,-.25f,-.25f);
	glVertex3f(0,0,0);
	glVertex3f(.6f,.25f,-.25f);

	glNormal3f(-.385f,-.923f,0);
	glVertex3f(.6f,-.25f,.25f);
	glVertex3f(0,0,0);
	glVertex3f(.6f,-.25f,-.25f);

	glEnd();

}