#include "IntentionGenerator.h"
#include "Agent.h"

#ifndef max
#define max(a,b) (((a)>(b))?(a):(b))
#endif

IntentionGenerator::IntentionGenerator()
{
	agent = NULL;
	adjustments.resize(6);
	Reset();
}

IntentionGenerator::~IntentionGenerator()
{
}

void
IntentionGenerator::Reset()
{
	adjustments[FACING] = 1;
	adjustments[IN_REACH] = 1;
	adjustments[FEELING_GOOD] = 1;
	adjustments[INCOMING] = 1;
	adjustments[OPENING] = 1;
	adjustments[CORNERED] = 1;
}

void
IntentionGenerator::SetAgent(Agent * a)
{
	agent = a;
}

Intention
IntentionGenerator::GenerateIntention()
{
	path.clear();
	pathchoices.clear();

	if (ChoosePath(FACING))
	{
		if (ChoosePath(IN_REACH))
		{
			if (ChoosePath(INCOMING))
			{
				return EVADE;
			}
			else	// Not incoming
			{
				if (ChoosePath(OPENING))
				{
					return ATTACK;
				}
				else	// No opening
				{
					if (ChoosePath(CORNERED))
					{
						return ESCAPE_ROPES;
					}
					else	// Not cornered
					{
						if (ChoosePath(FEELING_GOOD))
						{
							return ATTACK;
						}
						else	// Not feeling good
						{
							return DEFEND;
						}
					}
				}
			}
		}
		else	// Not in reach
		{
			if (ChoosePath(FEELING_GOOD))
			{
				return APPROACH;
			}
			else	// Not feeling good
			{
				return RECOVER;
			}
		}
	} else {	// Not facing
		return FACE;
	}
}

void
IntentionGenerator::ScorePath(float score)
{
	float thisscore;

	for (int i = path.size() - 1; i >= 0; --i)
	{
		
		if (pathchoices[i])
			thisscore = score;
		else
			thisscore = 1 / score;

		adjustments[path[i]] *= thisscore;
		score = (1.0f + score) / 2.0f;
	}

	/*for (int i = path.size() - 1; i >= 0; --i)
	{
		switch (path[i])
		{
		case FACING:
			printf(" f");
			break;
		case IN_REACH:
			printf(" r");
			break;
		case FEELING_GOOD:
			printf(" g");
			break;
		case INCOMING:
			printf(" i");
			break;
		case OPENING:
			printf(" o");
			break;
		case CORNERED:
			printf(" c");
			break;
		}
	}*/
}

bool
IntentionGenerator::WriteProfile(const char * filename)
{
	FILE * file;

	file = fopen(filename, "w"); 
	if (file == NULL)
		return false;

	fprintf( file, "%f\n%f\n%f\n%f\n%f\n%f",
	         GetAdjustment(FACING),
	         GetAdjustment(IN_REACH),
	         GetAdjustment(FEELING_GOOD),
	         GetAdjustment(INCOMING),
	         GetAdjustment(OPENING),
	         GetAdjustment(CORNERED) );
	fclose(file);

	return true;
}

bool
IntentionGenerator::LoadProfile(const char * filename)
{
	FILE * file;

	file = fopen(filename, "r"); 
	if (file == NULL)
		return false;

	fscanf( file, "%f\n%f\n%f\n%f\n%f\n%f",
	        &adjustments[FACING],
			&adjustments[IN_REACH],
			&adjustments[FEELING_GOOD],
			&adjustments[INCOMING],
			&adjustments[OPENING],
			&adjustments[CORNERED] );

	fclose(file);

	return true;
}

float
IntentionGenerator::GetAdjustment(Condition adj)
{
	return adjustments[adj];
}

bool
IntentionGenerator::EvalCond(Condition cond)
{
	float conf, adjust, r;

	switch (cond)
	{
	case FACING:
		conf = Facing();
		break;
	case IN_REACH:
		conf = InReach();
		break;
	case FEELING_GOOD:
		conf = FeelingGood();
		break;
	case INCOMING:
		conf = Incoming();
		break;
	case OPENING:
		conf = Opening();
		break;
	case CORNERED:
		conf = Cornered();
		break;
	default:
		printf("Error in eval cond in intention generator\n");
		return false;
	}

	if (conf != 1 && conf != 0)
	{
		adjust = adjustments[cond];
		if (adjust <= 1)
			conf *= adjust;
		else
			conf = 1 - ((1 - conf) / adjust);
	}

	r = (float) rand() / (float) RAND_MAX;

	return (conf > r);
}

bool
IntentionGenerator::ChoosePath(Condition cond)
{
	bool choice;

	path.push_back(cond);
	choice = EvalCond(cond);
	pathchoices.push_back(choice);

	return choice;
}

float
IntentionGenerator::Facing()
{
	float angle, maxangle;

	maxangle = 10.0f;
	angle = abs(agent->GetPerception()->OpponentAngle());

	if (angle > maxangle)
		return 0.0f;

	return max(0.0f, cos(angle * 3.14159f / (2 * maxangle)));
}

float
IntentionGenerator::InReach()
{
	float dist, reach;

	reach = 1.5f;
	dist = agent->GetPerception()->OpponentDistance();
	if (dist > 2 * reach)
		return 0.0f;

	return max(0.0f, cos(1.0472f * dist / reach));
}

float
IntentionGenerator::FeelingGood()
{
	return agent->GetState()->GetEnergy();
}

float
IntentionGenerator::Incoming()
{
	float popangle = 0, skillmod = 0.6f;

	if (abs(agent->GetPerception()->OpponentAngle() < 45))
		popangle = agent->GetOpponent()->GetPerception()->OpponentAngle();

	if (abs(popangle) > 15)
		return 0.0f;

	if (agent->GetActor()->GetAttribute("skilled") > 0.5f)
		skillmod = 1.0f;

	if (agent->GetActor()->GetAttribute("defensive") > 0.5f)
	{
		if ( (agent->GetPerception()->JointVisible("lhand") &&
		      agent->GetOpponent()->GetAction() == "jab") ||
		     ( agent->GetPerception()->JointVisible("rhand") &&
		       (agent->GetOpponent()->GetAction() == "cross" ||
		        agent->GetOpponent()->GetAction() == "hook" ||
		        agent->GetOpponent()->GetAction() == "upper_cut" ||
		        agent->GetOpponent()->GetAction() == "feint")
		     )
		   )
			return (0.9f * skillmod);
		else
			return (0.45f * skillmod);
	}

	if (agent->GetActor()->GetAttribute("offensive") > 0.5f)
	{
		if ( (agent->GetPerception()->JointVisible("lhand") &&
		      agent->GetOpponent()->GetAction() == "jab") ||
		     ( agent->GetPerception()->JointVisible("rhand") &&
		       (agent->GetOpponent()->GetAction() == "cross" ||
		        agent->GetOpponent()->GetAction() == "hook" ||
		        agent->GetOpponent()->GetAction() == "upper_cut" ||
		        agent->GetOpponent()->GetAction() == "feint")
		     )
		   )
			return (0.6f * skillmod);
		else
			return (0.3f * skillmod);
	}

	return 0;
}

float
IntentionGenerator::Opening()
{
	float popangle = 0;
	
	if (abs(agent->GetPerception()->OpponentAngle() < 45))
		popangle = agent->GetOpponent()->GetPerception()->OpponentAngle();
	else
		return 0;

	if (abs(popangle) > 45)
		return 1.0f;
	else if (abs(popangle) > 15)
		return (abs(popangle) - 15) / 30;

	if (agent->GetActor()->GetAttribute("defensive") > 0.5f)
	{
		if (agent->GetOpponent()->GetAction() == "recover")
			return 0.4f;
		else
			return 0.2f;
	}

	if (agent->GetActor()->GetAttribute("offensive") > 0.5f)
	{
		if (agent->GetOpponent()->GetAction() == "recover")
			return 0.9f;
		else
			return 0.5f;
	}

	return 0;
}

float
IntentionGenerator::Cornered()
{
	Vector3 pos;

	agent->GetPosition(pos);
	if (pos[0] > 3 || pos [0] < -3 || pos[2] > 3 || pos[2] < -3)
	{
		return max(abs(pos[0]) - 3, abs(pos[2]) - 3);
	}
	else
	{
		return 0;
	}
}