#include "KeyboardAgent.h"

KeyboardAgent::KeyboardAgent()
{
	behavioralmode = false;
	debugcolorr = 0.0;
	debugcolorg = 0.5;
	debugcolorb = 1.0;
	savedkey = ' ';	// Flag for no saved key
};

KeyboardAgent::~KeyboardAgent()
{
}

void
KeyboardAgent::HandleCharInput(char c)
{
	if (behavioralmode)	// Key bindings in BR control mode
	{
		switch (c)
		{
		case 'f':
			if (br.PlayIntention(FACE))
				thoughts[0] = "* Facing opponent";
			break;
		case 'r':
			if (br.PlayIntention(RECOVER))
				thoughts[0] = "* Recovering";
			break;
		case 'w':
			if (br.PlayIntention(APPROACH))
				thoughts[0] = "* Approaching";
			break;
		case 'e':
			if (br.PlayIntention(EVADE))
				thoughts[0] = "* Evading";
			break;
		case 'a':
			if (br.PlayIntention(ATTACK))
				thoughts[0] = "* Attacking!";
			break;
		case 'd':
			if (br.PlayIntention(DEFEND))
				thoughts[0] = "* Defending";
			break;
		case 'x':
			if (br.PlayIntention(ESCAPE_ROPES))
				thoughts[0] = "* Escaping ropes";
			break;
		default:
			break;
		}
	}
	else	// Key bindings in direct animation control
	{
		switch (c)
		{
		case 'w':
			if (PlayAction("step_forward"))
				thoughts[0] = "Stepping forward";
			break;
		case 's':
			if (PlayAction("step_back"))
				thoughts[0] = "Stepping back";
			break;
		case 'a':
			if (PlayAction("turn_left"))
				thoughts[0] = "Turning left";
			break;
		case 'd':
			if (PlayAction("turn_right"))
				thoughts[0] = "Turning right";
			break;
		case 'z':
			if (PlayAction("step_left"))
				thoughts[0] = "Stepping left";
			break;
		case 'x':
			if (PlayAction("step_right"))
				thoughts[0] = "Stepping right";
			break;
		case 'j':
			if (PlayAction("jab"))
				thoughts[0] = "Jab!";
			break;
		case 'c':
			if (PlayAction("cross"))
				thoughts[0] = "Cross!";
			break;
		case 'h':
			if (PlayAction("hook"))
				thoughts[0] = "Hook!";
			break;
		case 'u':
			if (PlayAction("upper_cut"))
				thoughts[0] = "Upper cut!";
			break;
		case 'b':
			if (PlayAction("block"))
				thoughts[0] = "Blocking";
			break;
		case 'f':
			if (PlayAction("feint"))
				thoughts[0] = "Feinting";
			break;
		case '[':
			if (PlayAction("dodge_left"))
				thoughts[0] = "Dodging left";
			break;
		case ']':
			if (PlayAction("dodge_right"))
				thoughts[0] = "Dodging right";
			break;
		case '\\':
			if (PlayAction("hold_glove"))
				thoughts[0] = "Holding out glove";
			break;
		case ' ':
			if (PlayAction("take_hit"))
				thoughts[0] = "I've been hit!";
			break;
		default:
			break;
		}
	}
}

void KeyboardAgent::ScoreAction(ActionTerminator at)
{
	state.FinishAction(at);
}

void 
KeyboardAgent::Think()
{
	for (unsigned int i = 0; i < events.size(); i++)
	{
		if (events[i].type == InputEvent::KeyDown)
		{
			switch (events[i].ID)	// Constant key bindings
			{
			case '`':
				behavioralmode = !behavioralmode;
				if (behavioralmode)
					thoughts[0] = "Intention input on";
				else
					thoughts[0] = "Animation input on";
				break;
			case '1':
				thoughts[0] = "Going amateur";
				actor.SetAttribute("amateur",1);
				actor.SetAttribute("skilled",0);
				actor.SetAttribute("tired",0);
				actor.SetAttribute("energetic",0);
				actor.SetAttribute("defensive",0);
				actor.SetAttribute("offensive",0);
				break;
			case '2':
				thoughts[0] = "Going skilled";
				actor.SetAttribute("amateur",0);
				actor.SetAttribute("skilled",1);
				actor.SetAttribute("tired",0);
				actor.SetAttribute("energetic",0);
				actor.SetAttribute("defensive",0);
				actor.SetAttribute("offensive",0);
				break;
			case '3':
				thoughts[0] = "Going tired";
				actor.SetAttribute("amateur",0);
				actor.SetAttribute("skilled",0);
				actor.SetAttribute("tired",1);
				actor.SetAttribute("energetic",0);
				actor.SetAttribute("defensive",0);
				actor.SetAttribute("offensive",0);
				break;
			case '4':
				thoughts[0] = "Going energetic";
				actor.SetAttribute("amateur",0);
				actor.SetAttribute("skilled",0);
				actor.SetAttribute("tired",0);
				actor.SetAttribute("energetic",1);
				actor.SetAttribute("defensive",0);
				actor.SetAttribute("offensive",0);
				break;
			case '5':
				thoughts[0] = "Going defensive";
				actor.SetAttribute("amateur",0);
				actor.SetAttribute("skilled",0);
				actor.SetAttribute("tired",0);
				actor.SetAttribute("energetic",0);
				actor.SetAttribute("defensive",1);
				actor.SetAttribute("offensive",0);
				break;
			case '6':
				thoughts[0] = "Going offensive";
				actor.SetAttribute("amateur",0);
				actor.SetAttribute("skilled",0);
				actor.SetAttribute("tired",0);
				actor.SetAttribute("energetic",0);
				actor.SetAttribute("defensive",0);
				actor.SetAttribute("offensive",1);
				break;
			case ',':
				thoughts[0] = "Small actions";
				actor.SetAttribute("small",1);
				actor.SetAttribute("big",0);
				break;
			case '.':
				thoughts[0] = "Big actions";
				actor.SetAttribute("small",0);
				actor.SetAttribute("big",1);
				break;
			default:
				if (!actioninterruptible && !actor.IsIdle())
					savedkey = events[i].ID;
				else
					HandleCharInput(events[i].ID);
				break;
			}
		}
	}
	events.clear();


	if (actor.IsIdle())
	{
		if (savedkey != ' ')
		{
			HandleCharInput(savedkey);
			savedkey = ' ';
		}
		else if (PlayAction("idle"))
		{
			thoughts[0] = "Idling";
		}
	}

	thoughts[1] = actor.GetCurrentAnimation();

/*	float opangle = perception.OpponentAngle();

	char str[50];
	sprintf(str, "%f", opangle);

	thoughts[2] = str;*/
}