#include "Perception.h"
#include "Agent.h"
#include "Matrix4x4.h"
#include "Vector3.h"

Perception::Perception()
{
	agent = NULL;
	lastangle = 0;
	lastdistance = 5;
}

Perception::~Perception()
{
}

void
Perception::SetAgent(Agent * a)
{
	agent = a;
}

float
Perception::OpponentAngle()
{
	Matrix4x4 ophead, myhead;
	Vector3 op(0,0,0), me(0,0,0), my_z(0.1736f,0,0.9848f), op_dir, z_dir;
	float opangle;

	if (agent == NULL || agent->GetOpponent() == NULL)
		return 0;

	agent->GetOpponent()->GetActor()->GetJointXform("head", ophead);
	agent->GetActor()->GetJointXform("head", myhead);

	op.Mult(ophead);
	me.Mult(myhead);
	my_z.Mult(myhead);

	op_dir.Sub(op, me);
	z_dir.Sub(my_z, me);

	opangle = (atan2(op_dir[0],op_dir[2]) - atan2(z_dir[0],z_dir[2]))
		* 180.0f/3.14159f;

	if (opangle > 180) opangle -= 360;
	else if (opangle < -180) opangle += 360;

	lastangle = opangle;

	if (opangle > 60)
		return 60;
	if (opangle < -60)
		return -60;

	return opangle;
}

float
Perception::OpponentDistance()
{
	Matrix4x4 ophead, myhead;
	Vector3 op(0,0,0), me(0,0,0);

	if (agent == NULL || agent->GetOpponent() == NULL)
		return 0;

	if (lastangle > 45)
		return lastdistance;

	agent->GetOpponent()->GetActor()->GetJointXform("head", ophead);
	agent->GetActor()->GetJointXform("head", myhead);

	op.Mult(ophead);
	me.Mult(myhead);
	op.Sub(me);

	lastdistance = op.Magnitude();

	return lastdistance;
}

bool
Perception::JointVisible(const char * name)
{
	Matrix4x4 opjoint, myhead;
	Vector3 op(0,0,0), me(0,0,0), my_z(0.1736f,0,0.9848f), op_dir, z_dir;
	float dot;

	if (agent == NULL || agent->GetOpponent() == NULL)
		return false;

	agent->GetOpponent()->GetActor()->GetJointXform(name, opjoint);
	agent->GetActor()->GetJointXform("head", myhead);

	op.Mult(opjoint);
	me.Mult(myhead);
	my_z.Mult(myhead);

	op_dir.Sub(op, me);
	z_dir.Sub(my_z, me);

	op_dir.Normalize();
	z_dir.Normalize();

	dot = op_dir.Dot(z_dir);

	return (acos(dot) < 0.8f);	// A little more than 45 degrees
}

float
Perception::GetLastDistance()
{
	return lastdistance;
}

float
Perception::GetLastAngle()
{
	return lastangle;
}