#include "SkelCharInst.h"
#include <Axis3D.h>
SkelCharInst::SkelCharInst()
{
	//should never be here!
	skelly = NULL;
	interleavedArray = NULL;
	useInterleaved = false;
}

SkelCharInst::~SkelCharInst()
{
	skelly = NULL;
	free(interleavedArray);
}

SkelCharInst::SkelCharInst(SkelChar & skel,bool inter)
{
	useInterleaved = inter;
	skelly = &skel;
	interleavedArray = NULL;
	Init();
}



void
SkelCharInst::Init()
{
	if (useInterleaved)
		interleavedArray = (GLfloat*)malloc(skelly->numPolys * 6 * 3 *4);

	norms.resize(skelly->numNormals);
	verts.resize(skelly->numVerts);
	joints.resize(skelly->numJoints);
	for (unsigned int i  =0; i < skelly->numJoints;i++)
	{
		joints[i].dirty = true;
		joints[i].Mfinal.Identity();
		joints[i].xform.Identity();
		joints[i].xTrans.Set(0,0,0);
		joints[i].xRot.Identity();
		joints[i].xScale.Set(1,1,1);

	}
	Deform();
}

//set a joint's transform
void 
SkelCharInst::SetJointTransform(const unsigned int joint, const Matrix4x4 & rot, const Vector3 & scale, const Vector3 & trans)
{
	if (joint < skelly->numJoints)
	{
		joints[joint].dirty = true;
		joints[joint].xRot = rot;
		joints[joint].xScale = scale;
		joints[joint].xTrans = trans;
		Matrix4x4 temp;
		temp.Scale(scale);
		joints[joint].xform.Mult(rot,temp);
	}
}


void SkelCharInst::Draw()
{


	//glColor3f(0.5f,0.5f,0.5f);


	if (useInterleaved)
	{

		glBegin(GL_TRIANGLES);
		for (unsigned int i = 0; i < skelly->numPolys; i++)
		{
			glNormal3fv(&interleavedArray[6*3*i]);
			glVertex3fv(&interleavedArray[6*3*i+3]);
			glNormal3fv(&interleavedArray[6*3*i+6]);
			glVertex3fv(&interleavedArray[6*3*i+9]);
			glNormal3fv(&interleavedArray[6*3*i+12]);
			glVertex3fv(&interleavedArray[6*3*i+15]);

		}
		glEnd();

		//glInterleavedArrays( GL_N3F_V3F, 0, interleavedArray );
		//glDrawArrays(GL_TRIANGLES,0,skelly->numPolys);
	}
	else
	{
		glBegin(GL_TRIANGLES);
		for (unsigned int i = 0; i < skelly->numPolys; i++)
		{
			glNormal3fv(norms[skelly->polys[i].norms[0]].GetPtr());
			glTexCoord2fv(skelly->uvs[skelly->polys[i].uvs[0]].dat);
			glVertex3fv(verts[skelly->polys[i].verts[0]].GetPtr());
			glNormal3fv(norms[skelly->polys[i].norms[1]].GetPtr());
			glTexCoord2fv(skelly->uvs[skelly->polys[i].uvs[1]].dat);
			glVertex3fv(verts[skelly->polys[i].verts[1]].GetPtr());
			glNormal3fv(norms[skelly->polys[i].norms[2]].GetPtr());
			glTexCoord2fv(skelly->uvs[skelly->polys[i].uvs[2]].dat);
			glVertex3fv(verts[skelly->polys[i].verts[2]].GetPtr());

		}
		glEnd();
	
	}

	return;
	glDisable(GL_LIGHTING);
	//draw a stick-skeleton
	glColor3f(1,0,0);

	glBegin(GL_LINES);
	for (unsigned int i = 0; i < skelly->numJoints; i++)
	{
		if (skelly->joints[i].parent >= 0)
		{
			Vector3 vec(0,0,0);
			
			vec.Mult(joints[i].Mfinal);
			glVertex3fv(vec.GetPtr());

			vec.Set(0,0,0);
			vec.Mult(joints[skelly->joints[i].parent].Mfinal);
			glVertex3fv(vec.GetPtr());
		}

	}
	glEnd();

	for (unsigned int i = 0; i < skelly->numJoints; i++)
	{
		glPushMatrix();
		Matrix4x4 m = joints[i].Mfinal;
		m.Transpose();
		glMultMatrixf(m.GetPtr());
		glScalef(.1f,.1f,.1f);
		Axis3D::StaticDraw();

		glPopMatrix();
	}

	glEnable(GL_LIGHTING);


}



int 
SkelCharInst::GetNumJoints() const
{
	return skelly->numJoints;
}

void 
SkelCharInst::GetJointAngle(const unsigned int joint, Matrix4x4 & m) const
{
	if (joint < skelly->numJoints)
		m = joints[joint].xRot;
	else
		m.Identity();
}


void 
SkelCharInst::GetPose(Pose & p) const
{
	if (p.GetSize() != skelly->numJoints)
		p = Pose(skelly->numJoints);
	for (unsigned int i =0 ; i < skelly->numJoints; i++)
	{
		p.SetJointRot(i,joints[i].xRot);
	}
}

void 
SkelCharInst::SetPose(Pose & p)
{
	if (p.GetSize() != skelly->numJoints)
		return;

	for (unsigned int i =0 ; i < skelly->numJoints; i++)
	{
		p.GetJointRot(i).getMatrix(joints[i].xRot);
		joints[i].dirty = true;
		joints[i].xScale.Set(1,1,1);
		
		if (i == 0)
			joints[0].xTrans.Set(0,p.GetHeight(),0);
		else
			joints[i].xTrans.Set(0,0,0);
		
		Matrix4x4 t;
		t.Translate(joints[i].xTrans);
		joints[i].xform.Mult(joints[i].xRot,t);
	}
}


void SkelCharInst::Deform()
{
	static float val = 0;

	//update the final matrices based on the joint angles
	for (unsigned int i = 0; i < skelly->numJoints;i++)
	{

		Matrix4x4 relativeFinal;

		//get the final forward matrices 
		if (skelly->joints[i].parent >= 0)
		{

			Matrix4x4 ps;
			ps.Scale(	1.0f / joints[skelly->joints[i].parent].xScale.Get(0),
						1.0f / joints[skelly->joints[i].parent].xScale.Get(1),
						1.0f / joints[skelly->joints[i].parent].xScale.Get(2));
			
			relativeFinal.Mult(ps,skelly->joints[i].Mrelative);
			relativeFinal.PostMult(joints[i].xform);
			joints[i].Mfinal.Mult(joints[skelly->joints[i].parent].Mfinal,relativeFinal);
			
			
			if (joints[skelly->joints[i].parent].dirty)
				joints[i].dirty = true;
		}
		else
		{
			joints[i].Mfinal.Mult(skelly->joints[i].Mrelative,joints[i].xform);
		}
	}


	//now deform the vertices in the mesh
	Vector3 intvec;
	
	for (unsigned int i = 0; i < skelly->numVerts; i++)
	{
		bool isDirty = false;
		for (unsigned int j = 0; j < skelly->verts[i].numjoints; j++)
		{
			if (joints[skelly->verts[i].bone[j]].dirty)
			{
				isDirty = true;
				break;
			}
		}
		if (!isDirty)
			continue;

		verts[i].Set(0,0,0);

		for (unsigned int j = 0; j < skelly->verts[i].numjoints; j++)
		{
			intvec.Mult(skelly->verts[i].relpos[j],joints[skelly->verts[i].bone[j]].Mfinal);
			intvec.Mult(skelly->verts[i].weight[j]);
			verts[i].Add(intvec);
		}
		
	}
	
	
	//deform the normals
	for (unsigned int i =0; i < skelly->numNormals; i++)
	{
		bool isDirty = false;
		if (skelly->norms[i].vertex < 0)
		{
			//printf("%d\n",i);
			continue;
		}
		
		for (unsigned int j = 0; j < skelly->verts[skelly->norms[i].vertex].numjoints; j++)
		{
			if (joints[skelly->verts[skelly->norms[i].vertex].bone[j]].dirty)
			{
				isDirty = true;
				break;
			}
		}
		if (!isDirty)
			continue;

		norms[i].Set(0,0,0);
		
		for (unsigned int j = 0; j < skelly->verts[skelly->norms[i].vertex].numjoints; j++)
		{
			intvec.MultRot(skelly->norms[i].relorient[j],joints[skelly->verts[skelly->norms[i].vertex].bone[j]].Mfinal);
			intvec.Mult(skelly->verts[skelly->norms[i].vertex].weight[j]);
			norms[i].Add(intvec);
		}
		
		//norms[i].Normalize(); //not sure if this is needed
		
	}

	for (unsigned int i = 0; i < skelly->numJoints; i++)
		joints[i].dirty = false;

	if (useInterleaved)
	{
		for (unsigned int i = 0; i < skelly->numPolys; i++)
		{

			memcpy(&interleavedArray[i*6*3 ], norms[skelly->polys[i].norms[0]].GetPtr(),3*4);
			memcpy(&interleavedArray[i*6*3+3], verts[skelly->polys[i].verts[0]].GetPtr(),3*4);
			memcpy(&interleavedArray[i*6*3+6 ], norms[skelly->polys[i].norms[1]].GetPtr(),3*4);
			memcpy(&interleavedArray[i*6*3+9], verts[skelly->polys[i].verts[1]].GetPtr(),3*4);
			memcpy(&interleavedArray[i*6*3+12 ], norms[skelly->polys[i].norms[2]].GetPtr(),3*4);
			memcpy(&interleavedArray[i*6*3+15], verts[skelly->polys[i].verts[2]].GetPtr(),3*4);
		}
	}

}

void 
SkelCharInst::GetJointXform(const unsigned int joint, Matrix4x4 &  m)
{
	if (joint >= skelly->numJoints)
		return;
	GetFullXform(m);
	m.PostMult(joints[joint].Mfinal);
}

void 
SkelCharInst::GetJointXform(const char * name, Matrix4x4 &  m)
{
	GetJointXform(skelly->JointIndex(name),m);
}
