#include <Track.h>

Track::Track()
{
	index =0;
}

Track::~Track()
{
}

QTrack::QTrack()
{
}

QTrack::~QTrack()
{
}

VTrack::VTrack()
{
}

VTrack::~VTrack()
{
}

STrack::STrack()
{
}

STrack::~STrack()
{
}

void
Track::SetTime(float time)
{
	if (times.size() <= 0)
		return;
	
	ctime = time;

	//look forward from current position
	while ( (index < times.size()) && ( times[index] < time))
		index++;
	if (index >= times.size())
		index = (int)times.size()-1;
	//look backward from current position
	while ( (index >= 0) && ( times[index] > time))
		index--;
	//we end up pointing at the index just smaller than the value of time
}


Vector3
VTrack::GetValue()
{
	if (times.size() <= 0)
		return Vector3();

	//back before the start of the track
	if ((index == 0) && (ctime <= times[0]))
		return vals[0];
	//past the end of the track
	else if ( (index >= times.size()-1))
		return vals[vals.size()-1];
	//time for some linear interolation!
	else
	{
		float t1 = times[index];
		float t2 = times[index+1];
		float blend = (ctime - t1) / (t2 - t1);
		Vector3 retval; 
		retval.Interpolate(vals[index],vals[index+1],blend);
		return retval;
	}
}

Quaternion
QTrack::GetValue()
{
	if (times.size() <= 0)
		return Quaternion();

	//back before the start of the track
	if ((index == 0) && (ctime <= times[0]))
		return vals[0];
	//past the end of the track
	else if ( (index >= times.size()-1))
		return vals[vals.size()-1];
	//time for some linear interolation!
	else
	{
		float t1 = times[index];
		float t2 = times[index+1];
		float blend = (ctime - t1) / (t2 - t1);
		Quaternion retval; 
		retval.slerp(vals[index],vals[index+1],blend);
		return retval;
	}
}

float
STrack::GetValue()
{
	if (times.size() <= 0)
		return 0;

	//back before the start of the track
	if ((index == 0) && (ctime <= times[0]))
		return vals[0];
	//past the end of the track
	else if ( (index >= times.size()-1))
		return vals[vals.size()-1];
	//time for some linear interolation!
	else
	{
		float t1 = times[index];
		float t2 = times[index+1];
		float blend = (ctime - t1) / (t2 - t1);
		return (1.0f-blend)*vals[index]+blend*vals[index+1];
	}
}

void
VTrack::AddValue(float t, Vector3& v)
{
	//assume that things are added in order
	if (times.size() && times[times.size()-1] > t)
		return;
	times.push_back(t);
	vals.push_back(v);
}

void
QTrack::AddValue(float t, Quaternion & v)
{
	//assume that things are added in order
	if (times.size() && times[times.size()-1] > t)
		return;
	times.push_back(t);
	vals.push_back(v);
}

void
STrack::AddValue(float t, float v)
{
	//assume that things are added in order
	if (times.size() && times[times.size()-1] > t)
		return;
	times.push_back(t);
	vals.push_back(v);
}

